"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Connection = void 0;
const constants_1 = require("./constants");
const logger_1 = __importStar(require("./logger"));
class Connection {
    constructor(onMessage, onBinaryData, onError, onClose, logger) {
        this.connected = false;
        this.onMessage = onMessage;
        this.onBinaryData = onBinaryData;
        this.onError = onError;
        this.onClose = onClose;
        this.logger = (logger) ? logger : new logger_1.default();
        if (!logger) {
            this.logger.setPrefix(() => "[Connection]");
            this.logger.setVerbosity(logger_1.Verbosity.INFO);
        }
    }
    connect(options) {
        this.ws = new WebSocket(options.url);
        this.ws.binaryType = 'arraybuffer';
        const that = this;
        return new Promise(function (resolve, reject) {
            if (options.timeout !== undefined && options.timeout > 0) {
                that.connectionTimeout = window.setTimeout(function () {
                    if (that.ws.readyState === 0) {
                        that.logger.warn("WS connection timeout");
                        that.ws.close();
                    }
                }, options.timeout);
            }
            that.ws.onerror = function (e) {
                if (that.connected) {
                    that.onError(e);
                }
                else {
                    reject(e);
                }
            };
            that.ws.onclose = function (e) {
                that.logger.debug("WebSocket connection closed, reason " + e.reason);
                if (that.connected) {
                    that.onClose(e);
                }
                else {
                    reject(e);
                }
            };
            that.ws.onopen = function () {
                clearTimeout(that.connectionTimeout);
                let cConfig = {
                    appKey: options.appName,
                    mediaProviders: ["WebRTC"],
                    keepAlive: true,
                    authToken: null,
                    clientVersion: "0.5.28",
                    clientOSVersion: window.navigator.appVersion,
                    clientBrowserVersion: window.navigator.userAgent,
                    custom: options.custom
                };
                //connect to REST App
                that.send("connection", cConfig);
            };
            that.ws.onmessage = function (event) {
                let name;
                let msg;
                if (event.data instanceof ArrayBuffer) {
                    name = constants_1.InternalApi.BINARY_DATA;
                }
                else {
                    let obj = JSON.parse(event.data);
                    name = obj.message;
                    msg = obj.data;
                }
                switch (name) {
                    case "ping":
                        that.send("pong", null);
                        break;
                    case "getUserData":
                        that.connected = true;
                        resolve(msg[0]);
                        break;
                    case "binaryData":
                        that.onBinaryData(name, event.data);
                        break;
                    default:
                        that.onMessage(name, msg);
                }
            };
        });
    }
    send(message, data) {
        this.logger.info("==>", message, data);
        if (this.ws) {
            this.ws.send(JSON.stringify({
                message: message,
                data: [data]
            }));
        }
    }
    ;
    sendBinaryData(data) {
        this.logger.info("==>", data.size);
        if (this.ws) {
            this.ws.send(data);
        }
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.ws) {
                const self = this;
                return new Promise(function (resolve) {
                    self.ws.onclose = function (e) {
                        self.logger.debug("WebSocket connection closed, reason " + e.reason);
                        if (self.connected) {
                            self.onClose(e);
                        }
                        resolve();
                    };
                    self.ws.close();
                });
            }
        });
    }
}
exports.Connection = Connection;
//# sourceMappingURL=connection.js.map